#include <iostream>
#include <vector>
#include "svg.h"

void //      SVG
svg_begin(double width, double height) {
    std::cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    std::cout << "<svg ";
    std::cout << "width='" << width << "' ";
    std::cout << "height='" << height << "' ";
    std::cout << "viewBox='0 0 " << width << " " << height << "' ";
    std::cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void
svg_end() {
    std::cout << "</svg>\n";
}

void svg_text(double left, double baseline, std::string text){
    std::cout << "<text x='" << left << "' y='" << baseline << "' > '" << text << " </text>";

}

void svg_rect(double x, double y, double width, double height){
    std::cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << " ' stroke='black' fill='#D499E6' ></rect>";
}

void
show_histogram_svg(const std::vector<std::size_t>& bins){
    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto BLOCK_WIDTH = 10;
    const auto MAX_ASTERISK = IMAGE_WIDTH - TEXT_WIDTH;

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);

    double top = 0;
    std::size_t maxbin=bins[0];
    for (std::size_t i=0; i < bins.size(); i++){
        if (maxbin < bins[i]){
            maxbin=bins[i];
        }
    }
    if (maxbin<=76){
        for (std::size_t i=0; i < bins.size(); i++) {
            double bin_width = BLOCK_WIDTH * bins[i];
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
        svg_end();
    } else {
        for (std::size_t i=0; i < bins.size(); i++) {
            double bin_width= MAX_ASTERISK * (static_cast<double>(bins[i]) / maxbin);
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
        svg_end();
    }
}

void show_histogram_svg_vertical(const std::vector<std::size_t>& bins) {
    const auto IMAGE_WIDTH = 300;
    const auto IMAGE_HEIGHT = 400;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 30;
    const auto BIN_WIDTH = 30;
    const auto BLOCK_HEIGHT = 10;
    const auto MAX_ASTERISK = IMAGE_HEIGHT - TEXT_WIDTH;

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);

    double left = 0;
    std::size_t maxbin = bins[0];
    for (std::size_t i = 0; i < bins.size(); i++) {
        if (maxbin < bins[i]) {
            maxbin = bins[i];
        }
    }
    if (maxbin <= 76) {
        for (std::size_t i = 0; i < bins.size(); i++) {
            double bin_height = BLOCK_HEIGHT * bins[i];
            svg_text(left, TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(left, TEXT_WIDTH, BIN_WIDTH, bin_height);
            left += BIN_WIDTH;
        }
        svg_end();
    } else {
        for (std::size_t i = 0; i < bins.size(); i++) {
            double bin_height = MAX_ASTERISK * (static_cast<double>(bins[i]) / maxbin);
            svg_text(left, TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(left, TEXT_WIDTH, BIN_WIDTH, bin_height);
            left += BIN_WIDTH;
        }
        svg_end();
    }
}
